/*
 * Copyright (c) 2009 Zauber S.A.  -- All rights reserved
 */
package ar.com.zauber.tools.crono.web.taglibs;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.web.servlet.tags.HtmlEscapingAwareTag;
import org.springframework.web.servlet.tags.RequestContextAwareTag;
import org.springframework.web.util.ExpressionEvaluationUtils;

import ar.com.zauber.commons.dao.exception.NoSuchEntityException;

/**
 * Clase para resolver mensajes en los JSP
 * 
 * @author Juan Almeyda
 * @since Oct 9, 2009
 */

@SuppressWarnings("serial")
public class MessageTagLib extends HtmlEscapingAwareTag {
    
    /** el texto correspondiente al archivo de properties*/
    private String code;
    /** parametros con los que completo el mensaje*/
    private List<String> values = new ArrayList<String>();

    public final void setCode(final String code) {
        this.code = code;
    }
    
    public final List<String> getParameters() {
        return values;
    }
    
    /** @see RequestContextAwareTag#doStartTagInternal() */
    protected final int doStartTagInternal() {
        values.clear();
        return EVAL_BODY_INCLUDE;
    }
    
    /** @see TagSupport#doEndTag()*/
    public final int doEndTag() throws JspException {
        try {
            String msg = resolveMessage();
            writeMessage(msg);
            return EVAL_PAGE;
        } catch (NoSuchEntityException ex) {
            throw new JspTagException(ex.getMessage());
        } catch (IOException ex) {
             throw new JspException(ex.getMessage());
        }
    }

    /** @return el texto del mensaje del archivo de properties con los parametros 
     *  ya insertados*/
    protected final String resolveMessage() throws JspException, 
            NoSuchMessageException {
        MessageSource messageSource = getMessageSource();
        if (messageSource == null) {
            throw new JspTagException("No corresponding MessageSource found");
        }
        String resolvedCode = ExpressionEvaluationUtils.evaluateString("code", 
                this.code, pageContext);
        Object[] argumentsArray = values.toArray();
        return messageSource.getMessage(resolvedCode, argumentsArray, 
                getRequestContext().getLocale());
    }

    /** Use the current RequestContext's application context as MessageSource.*/
    protected final MessageSource getMessageSource() {
        return getRequestContext().getMessageSource();
    }
    
    /**
     * Write the message to the page.
     * <p>Can be overridden in subclasses, e.g. for testing purposes.
     * @param msg the message to write
     * @throws IOException if writing failed
     */
    protected final void writeMessage(final String msg) throws IOException {
        pageContext.getOut().write(String.valueOf(msg));
    }
}